"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Api = void 0;
const common_1 = require("@tencent-serverless-framework/common");
const _1 = require(".");
const formatter_1 = require("./formatter");
const logger_1 = require("@jsonlog/logger");
const logger = logger_1.getLogger({
    name: 'APIGW - API',
    middlewares: [logger_1.printToConsole],
});
/**
 * 清除请求中的无关参数
 *
 * @param req 请求
 * @returns 清除后的请求
 */
function cleanReq(req) {
    delete req.EnableCors;
    delete req.Created;
    return req;
}
/** API 网关组件 API 子模块 */
class Api {
    /**
     * 构造函数
     *
     * @param apigw API 网关组件（依赖注入）
     */
    constructor(apigw) {
        this.apigw = apigw;
    }
    /**
     * 部署 API 网关 API列表
     *
     * @param inputs 部署 API 网关输入
     * @returns 部署 API 网关输出
     */
    async apiDeployer(inputs) {
        const { serviceId, environment, apiList = [], oldList, apiConfig, isOauthApi } = inputs;
        // if exist in state list, set created to be true
        const [exist] = oldList.filter((item) => item.method.toLowerCase() === apiConfig.method.toLowerCase() &&
            item.path === apiConfig.path);
        if (exist) {
            apiConfig.apiId = exist.apiId;
            apiConfig.created = exist.created;
            if (isOauthApi) {
                apiConfig.authRelationApiId = exist.authRelationApiId;
            }
        }
        if (isOauthApi && !apiConfig.authRelationApiId) {
            // find reletive oauth api
            const { authRelationApi } = apiConfig;
            if (authRelationApi) {
                const [relativeApi] = apiList.filter((item) => item.method.toLowerCase() === authRelationApi.method.toLowerCase() &&
                    item.path === authRelationApi.path);
                if (relativeApi) {
                    apiConfig.authRelationApiId = relativeApi.apiId;
                }
            }
        }
        const curApi = await this.createOrUpdateApi({
            serviceId,
            environment,
            endpoint: apiConfig,
            created: exist && exist.created,
        });
        logger.info({ msg: _1.t('部署 API {{apiName}} 成功', { apiName: curApi.apiName }) });
        return curApi;
    }
    /**
     * 移除 API 网关 API
     *
     * @param inputs 移除 API 网关 API输入参数
     */
    async apiRemover(inputs) {
        const { apiConfig, serviceId, environment } = inputs;
        // 1. remove usage plan
        if (apiConfig.usagePlan) {
            await this.apigw.usagePlan.removeOrUnbindUsagePlan({
                serviceId,
                environment,
                apiId: apiConfig.apiId,
                usagePlan: apiConfig.usagePlan,
            });
        }
        // 2. delete only apis created by serverless framework
        if (apiConfig.apiId && apiConfig.created === true) {
            logger.info({ msg: _1.t('移除 API {{apiId}} 成功', { apiId: apiConfig.apiId }) });
            await this.apigw.trigger.removeTrigger({
                serviceId,
                apiId: apiConfig.apiId,
            });
            try {
                await this.apigw.client.DeleteApi({
                    ApiId: apiConfig.apiId,
                    ServiceId: serviceId,
                });
            }
            catch (err) {
                // FIMXE: 有时候删除会有
                logger.warn({ msg: _1.t('发生错误：{{err}}', { err }) });
            }
        }
    }
    /**
     * 创建或更新 API 网关 API
     *
     * @param inputs 创建或更新 API 网关 API 输入
     * @returns 完成后的输出
     */
    async createOrUpdateApi(inputs) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
        const { serviceId, endpoint, environment, created } = inputs;
        // compatibility for secret auth config depends on auth & usagePlan
        const authType = endpoint.auth ? 'SECRET' : endpoint.authType || 'NONE';
        const businessType = endpoint.businessType || 'NORMAL';
        const output = {
            path: endpoint.path,
            method: endpoint.method,
            apiName: endpoint.apiName || 'index',
            created: true,
            authType: authType,
            businessType: businessType,
            isBase64Encoded: endpoint.isBase64Encoded === true,
        };
        if (endpoint.authRelationApiId) {
            output.authRelationApiId = endpoint.authRelationApiId;
        }
        const apiInputs = {
            protocol: (_a = endpoint.protocol) !== null && _a !== void 0 ? _a : 'HTTP',
            serviceTimeout: (_b = endpoint.serviceTimeout) !== null && _b !== void 0 ? _b : 15,
            responseType: (_c = endpoint.responseType) !== null && _c !== void 0 ? _c : 'HTML',
            apiName: (_d = endpoint.apiName) !== null && _d !== void 0 ? _d : 'index',
            apiBusinessType: (_e = endpoint.businessType) !== null && _e !== void 0 ? _e : 'NORMAL',
            serviceType: (_f = endpoint.serviceType) !== null && _f !== void 0 ? _f : 'SCF',
            serviceId: serviceId,
            apiDesc: endpoint.description,
            apiType: 'NORMAL',
            authType: authType,
            requestConfig: {
                path: endpoint.path,
                method: endpoint.method,
            },
            enableCORS: endpoint.enableCORS === true,
            isBase64Encoded: endpoint.isBase64Encoded === true,
            oauthConfig: endpoint.oauthConfig,
            authRelationApiId: endpoint.authRelationApiId,
        };
        formatter_1.marshalApiInput(endpoint, apiInputs);
        let apiDetail = undefined;
        if (endpoint.apiId) {
            apiDetail = await this.getApiById({ serviceId, apiId: endpoint.apiId });
        }
        if (!apiDetail) {
            apiDetail = await this.getApiByPathAndMethod({
                serviceId,
                path: endpoint.path,
                method: endpoint.method,
            });
        }
        if (apiDetail) {
            logger.info({
                msg: _1.t('API 方法={{method}}, 路径={{path}} 已经存在，ID 为 {{ApiId}}', {
                    method: endpoint.method,
                    path: endpoint.path,
                    ApiId: apiDetail.ApiId,
                }),
            });
            endpoint.apiId = apiDetail.ApiId;
            if (endpoint.isBase64Encoded && endpoint.isBase64Trigger) {
                apiInputs.isBase64Trigger = endpoint.isBase64Trigger;
                apiInputs.base64EncodedTriggerRules = endpoint.base64EncodedTriggerRules;
            }
            const modifyReq = {
                ...common_1.pascalCaseProperty(apiInputs),
                ApiId: endpoint.apiId,
                EnableCORS: apiInputs.enableCORS,
            };
            await this.apigw.client.ModifyApi(cleanReq(modifyReq));
            output.apiId = endpoint.apiId;
            output.created = !!created;
            output.internalDomain = apiDetail.InternalDomain || '';
            logger.info({ msg: _1.t('API {{apiId}} 已经更新', { apiId: output.apiId }) });
        }
        else {
            const createInputs = {
                ...common_1.pascalCaseProperty(apiInputs),
                Protocol: (_g = endpoint.protocol) !== null && _g !== void 0 ? _g : 'HTTP',
                ServiceTimeout: (_h = endpoint.serviceTimeout) !== null && _h !== void 0 ? _h : 15,
                ResponseType: (_j = endpoint.responseType) !== null && _j !== void 0 ? _j : 'HTML',
                ApiName: (_k = endpoint.apiName) !== null && _k !== void 0 ? _k : 'index',
                ApiBusinessType: (_l = endpoint.businessType) !== null && _l !== void 0 ? _l : 'NORMAL',
                EnableCORS: apiInputs.enableCORS,
            };
            delete createInputs.EnableCors;
            const createRes = await this.apigw.client.CreateApi(common_1.removeUndefined(createInputs));
            const { ApiId } = createRes.Result;
            output.apiId = ApiId;
            output.created = true;
            logger.info({ msg: _1.t('API {{ApiId}} 已经创建', { ApiId }) });
            const describeRes = await this.apigw.client.DescribeApi({
                ServiceId: serviceId,
                ApiId: output.apiId,
            });
            apiDetail = describeRes.Result;
            output.internalDomain = (_m = apiDetail === null || apiDetail === void 0 ? void 0 : apiDetail.InternalDomain) !== null && _m !== void 0 ? _m : '';
            if (endpoint.isBase64Encoded && endpoint.isBase64Trigger) {
                apiInputs.isBase64Trigger = endpoint.isBase64Trigger;
                apiInputs.base64EncodedTriggerRules = endpoint.base64EncodedTriggerRules;
            }
            const modifyReq = {
                ApiId: ApiId,
                ...common_1.pascalCaseProperty(apiInputs),
                EnableCORS: apiInputs.enableCORS,
            };
            await this.apigw.client.ModifyApi(cleanReq(modifyReq));
        }
        output.apiName = apiInputs.apiName;
        if (endpoint.usagePlan) {
            const usagePlan = await this.apigw.usagePlan.bindUsagePlan({
                apiId: output.apiId,
                serviceId,
                environment,
                usagePlanConfig: endpoint.usagePlan,
                authConfig: endpoint.auth,
            });
            output.usagePlan = usagePlan;
        }
        return output;
    }
    /**
     * 通过服务 ID 和 API ID 获取 API 网关 API 信息
     *
     * @param inputs 输入
     * @param inputs.serviceId API 网关服务 ID
     * @param inputs.apiId API 网关 API ID
     * @returns API 详情
     */
    async getApiById(inputs) {
        const { serviceId, apiId } = inputs;
        const res = await this.apigw.client.DescribeApi({
            ServiceId: serviceId,
            ApiId: apiId,
        });
        return res.Result;
    }
    /**
     * 根据路径和方法获取 API 网关 API 信息
     *
     * @param inputs 输入参数
     * @param inputs.serviceId API 网关服务 ID
     * @param inputs.path API 网关 API 接受路径
     * @param inputs.method API 网关 API 接受方法
     * @returns API 网关 API 详情
     */
    async getApiByPathAndMethod(inputs) {
        const { serviceId, path, method } = inputs;
        const res = await this.apigw.client.DescribeApisStatus({
            ServiceId: serviceId,
            Offset: 0,
            Limit: 100,
            Filters: [{ Name: 'ApiPath', Values: [path] }],
        });
        const { ApiIdStatusSet } = res.Result;
        let apiStatus = undefined;
        if (ApiIdStatusSet) {
            for (const item of ApiIdStatusSet) {
                if (item.Path === path && item.Method.toLowerCase() === method.toLowerCase()) {
                    apiStatus = item;
                }
            }
        }
        if (apiStatus) {
            const describeRes = await this.apigw.client.DescribeApi({
                ServiceId: serviceId,
                ApiId: apiStatus.ApiId,
            });
            return describeRes.Result;
        }
    }
}
exports.Api = Api;
