"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CustomDomain = exports.deepEqual = exports.isPrimitive = void 0;
const logger_1 = require("@jsonlog/logger");
const common_1 = require("@tencent-serverless-framework/common");
const _1 = require(".");
const formatter_1 = require("./formatter");
const logger = logger_1.getLogger({
    name: 'APIGW - CustomDomain',
    middlewares: [logger_1.printToConsole],
});
/**
 * 判断是否是值类型
 *
 * @param obj 要判断的对象
 * @returns 是否是值类型
 */
function isPrimitive(obj) {
    return obj !== Object(obj);
}
exports.isPrimitive = isPrimitive;
/**
 * 深度比较对象是否相等
 *
 * @param obj1 对象 1
 * @param obj2 对象 2
 * @returns 是否相等
 */
function deepEqual(obj1, obj2) {
    if (obj1 === obj2) {
        return true;
    }
    if (isPrimitive(obj1) || isPrimitive(obj2)) {
        return obj1 === obj2;
    }
    if (Object.keys(obj1).length !== Object.keys(obj2).length) {
        return false;
    }
    // compare objects with same number of keys
    for (const key in obj1) {
        if (!(key in obj2)) {
            return false;
        }
        if (!deepEqual(obj1[key], obj2[key])) {
            return false;
        }
    }
    return true;
}
exports.deepEqual = deepEqual;
/**
 * 获取自定义域名的字典
 *
 * @param domains 自定义域名数组
 * @returns 自定义域名字典
 */
function getCustomDomainFormattedDict(domains) {
    const domainDict = {};
    domains.forEach((d) => {
        var _a, _b, _c, _d;
        const pmDict = {};
        for (const pm of (_a = d.pathMappingSet) !== null && _a !== void 0 ? _a : []) {
            pmDict[pm.path] = pm.environment;
        }
        domainDict[d.domain] = {
            domain: d.domain,
            certificateId: (_b = d.certificateId) !== null && _b !== void 0 ? _b : '',
            protocols: formatter_1.formatProtocolString((_c = d.protocols) !== null && _c !== void 0 ? _c : ''),
            isDefaultMapping: d.isDefaultMapping === false ? false : true,
            pathMappingSetDict: pmDict,
            netType: (_d = d.netType) !== null && _d !== void 0 ? _d : 'OUTER',
        };
    });
    return domainDict;
}
/** API 网关组件自定义域名子模块 */
class CustomDomain {
    /**
     * 构造函数
     *
     * @param apigw API 网关组件（依赖注入）
     */
    constructor(apigw) {
        this.apigw = apigw;
    }
    /**
     * 获取当前 API 自定义域名字典
     *
     * @param serviceId API ID
     * @returns 自定义域名字典
     */
    async getCurrentCustomDomainsDict(serviceId) {
        var _a, _b;
        const res = await this.apigw.client.DescribeServiceSubDomains({
            ServiceId: serviceId,
        });
        const DomainSet = (_a = res.Result) === null || _a === void 0 ? void 0 : _a.DomainSet;
        const domainDict = {};
        for (const d of DomainSet !== null && DomainSet !== void 0 ? DomainSet : []) {
            const domain = {
                domain: d.DomainName,
                protocols: d.Protocol,
                certificateId: d.CertificateId,
                isDefaultMapping: d.IsDefaultMapping,
                netType: d.NetType,
                pathMappingSetDict: {},
            };
            const res_ = await this.apigw.client.DescribeServiceSubDomainMappings({
                ServiceId: serviceId,
                SubDomain: d.DomainName,
            });
            const PathMappingSet = (_b = res_.Result) === null || _b === void 0 ? void 0 : _b.PathMappingSet;
            PathMappingSet === null || PathMappingSet === void 0 ? void 0 : PathMappingSet.map((v) => {
                domain.pathMappingSetDict[v.Path] = v.Environment;
            });
            domainDict[domain.domain] = domain;
        }
        return domainDict;
    }
    /**
     * 解绑 API 网关所有由组件创建的自定义域名（通过查询 oldState）
     *
     * @param serviceId API 网关 ID
     * @param oldCustomDomains 状态中的自定义
     * @param currentDict 当前自定义域名字典
     * @param newDict 要绑定的自定义域名字典
     */
    async unbindCustomDomain(serviceId, oldCustomDomains, currentDict = {}, newDict = {}) {
        var _a, _b, _c, _d, _e, _f;
        const res = await this.apigw.client.DescribeServiceSubDomains({
            ServiceId: serviceId,
        });
        if (((_c = (_b = (_a = res.Result) === null || _a === void 0 ? void 0 : _a.DomainSet) === null || _b === void 0 ? void 0 : _b.length) !== null && _c !== void 0 ? _c : 0) > 0) {
            const DomainSet = (_e = (_d = res.Result) === null || _d === void 0 ? void 0 : _d.DomainSet) !== null && _e !== void 0 ? _e : [];
            // 解绑所有创建的自定义域名
            for (let i = 0; i < DomainSet.length; i++) {
                const domainItem = DomainSet[i];
                const domain = (_f = domainItem.DomainName) !== null && _f !== void 0 ? _f : '';
                // 当前绑定状态与新的绑定状态一致，不解绑
                if (currentDict[domain] && deepEqual(currentDict[domain], newDict[domain])) {
                    logger.info({
                        msg: _1.t('服务 {{servieId}} 的域名 {{DomainName} 未变化，跳过解绑', {
                            DomainName: domainItem.DomainName,
                            serviceId,
                        }),
                    });
                    continue;
                }
                for (let j = 0; j < oldCustomDomains.length; j++) {
                    // 只解绑由组件创建的域名
                    if (oldCustomDomains[j].subDomain === domainItem.DomainName) {
                        logger.info({
                            msg: _1.t('开始解绑自定义域名 {{DomainName}}，所属 API 网关 {{serviceId}}', {
                                serviceId,
                                DomainName: domainItem.DomainName,
                            }),
                        });
                        await this.apigw.client.UnBindSubDomain({
                            ServiceId: serviceId,
                            SubDomain: domainItem.DomainName,
                        });
                    }
                }
            }
        }
    }
    /**
     * 为 API 网关服务绑定自定义域名
     *
     * @param root0 输入参数
     * @param root0.serviceId API 网关服务 ID
     * @param root0.subDomain API 网关子域名
     * @param root0.inputs API 网关绑定自定义域名输入
     * @returns 绑定成功后的输出
     */
    async bindCustomDomain({ serviceId, subDomain, inputs, }) {
        var _a, _b, _c;
        logger.info({ msg: _1.t('正在绑定自定义域名 ...') });
        let { customDomains } = inputs;
        const { oldState = {} } = inputs;
        if (!customDomains) {
            // FIXME: 不存在自定义域名的时候，应该解绑之前绑定的域名
            customDomains = [];
        }
        const currentDict = await this.getCurrentCustomDomainsDict(serviceId);
        const newDict = getCustomDomainFormattedDict((_a = inputs.customDomains) !== null && _a !== void 0 ? _a : []);
        // 1. 解绑旧的自定义域名
        await this.unbindCustomDomain(serviceId, (_b = oldState === null || oldState === void 0 ? void 0 : oldState.customDomains) !== null && _b !== void 0 ? _b : [], currentDict, newDict);
        // 2. bind user config domain
        const customDomainOutput = [];
        if (customDomains && customDomains.length > 0) {
            logger.info({ msg: _1.t('开始绑定服务的自定义域名，服务 ID： {{serviceId}}', { serviceId }) });
            for (let i = 0; i < customDomains.length; i++) {
                const domainItem = customDomains[i];
                const domainProtocol = domainItem.protocols
                    ? formatter_1.formatProtocolString(domainItem.protocols)
                    : inputs.protocols;
                const domainInputs = {
                    serviceId,
                    subDomain: domainItem.domain,
                    netSubDomain: subDomain,
                    certificateId: domainItem.certificateId,
                    // default isDefaultMapping is true
                    isDefaultMapping: domainItem.isDefaultMapping === false ? false : true,
                    // if isDefaultMapping is false, should append pathMappingSet config
                    pathMappingSet: domainItem.pathMappingSet || [],
                    netType: (_c = domainItem.netType) !== null && _c !== void 0 ? _c : 'OUTER',
                    protocol: formatter_1.formatProtocolString(domainProtocol),
                    isForcedHttps: domainItem.isForcedHttps === true,
                };
                try {
                    const { domain } = domainItem;
                    // 当前状态与新的状态一致，不进行绑定
                    if (currentDict[domain] && deepEqual(currentDict[domain], newDict[domain])) {
                        logger.info({
                            msg: _1.t('服务 {{serviceId}} 的自定义域名 {{domain}} 没有变化，停止创建', {
                                serviceId,
                                domain,
                            }),
                        });
                        logger.info({
                            msg: _1.t('请在 {{domain}} 的 CNAME 中填入 {{subDomain}}', { domain, subDomain }),
                        });
                    }
                    else {
                        await this.apigw.client.BindSubDomain(common_1.pascalCaseProperty({
                            ...domainInputs,
                        }));
                        logger.info({ msg: _1.t('为服务 {{serviceId}} 绑定自定义域名 {{domain}} 成功') });
                        logger.info({ msg: _1.t('请在 {{domain}} 的 CNAME 中填入  {{subDomain}}') });
                    }
                    customDomainOutput.push({
                        isBinded: true,
                        created: true,
                        subDomain: domainItem.domain,
                        cname: subDomain,
                        url: `${domainProtocol.indexOf('https') !== -1 ? 'https' : 'http'}://${domainItem.domain}`,
                    });
                }
                catch (e) {
                    // User hasn't add cname dns record
                    if (e.code === 'FailedOperation.DomainResolveError') {
                        customDomainOutput.push({
                            isBinded: false,
                            subDomain: domainItem.domain,
                            cname: subDomain,
                            message: _1.t("您的自定义域名还未生效，请给域名 {{domain}} 添加 CNAME 记录 {{subDomain}}，等待解析生效后，再次运行 'sls deploy' 完成自定义域名的配置", { domain: domainItem.domain, subDomain }),
                        });
                    }
                    else {
                        throw e;
                    }
                }
            }
        }
        return customDomainOutput;
    }
}
exports.CustomDomain = CustomDomain;
